var GAME_SOUND = './sounds/background.ogg';
var GAME_SOUND_VOLUME = 0.05;
var CLICK_SOUND = './sounds/click.ogg';
var CLICK_SOUND_VOLUME = 0.80;
var FAIL_SOUND = './sounds/fail.ogg';
var FAIL_SOUND_VOLUME = 0.50;
var WIN_SOUND = './sounds/win.ogg';
var WIN_SOUND_VOLUME = 0.80;
var LOSE_SOUND = './sounds/lose.ogg';
var LOSE_SOUND_VOLUME = 0.80;
var GAME_CURSOR ='./images/game_images/cursor.png';
var GAME_ERROR_BG ='./images/game_images/error_bg.png';

window.requestAnimFrame = (function(){
  return  window.requestAnimationFrame       ||
          window.webkitRequestAnimationFrame ||
          window.mozRequestAnimationFrame    ||
          window.oRequestAnimationFrame      ||
          window.msRequestAnimationFrame     ||
          function( callback ){
            window.setTimeout(callback, 1000/ 60);
          };
})();

var Pizza = {

 /* Canvas */
    canvas : null,
    context : null,
    canvas_index : 0,
    
 /* Runtime */
    fps : 30,     
    runtimeID : null,
    timerID : null,

    /* Input */
    mouseX : null,
    mouseY : null,
    
    gameSound : null,
    clickSound : null,
    failSound : null,
    winSound : null,
    
    loader : null,
    player : null,
    
    difficulty  : null, 
    life : null,

    /* Start application */
    init : function(){
        this.setCanvasObject();
        this.load();
        this.attachEventListeners();   
    },
            
    /* Runtime */
    startRuntime : function(){
        
        this.runtimeID = new Array();
        var that = this;       
        setTimeout(function() { 
            that.runtimeID.push(setInterval(function(){
                Pizza.player.loop(0); 
            }, (1000 / this.fps)));
        }, 2000);
        setTimeout(function() {
            that.runtimeID.push(setInterval(function(){
                Pizza.player.loop(1); 
            }, (1000 / this.fps))); 
            Pizza.attachEventListeners();
        }, 2400);
        setTimeout(function() {
            that.runtimeID.push(setInterval(function(){
                Pizza.player.loop(2); 
            }, (1000 / this.fps))); 
        }, 3100);
    },
    
    stopRunTime : function(){
        
        for (var i = 0; i < this.runtimeID.length; i++) 
            clearInterval(this.runtimeID[i]);    
        this.removeEventListeners();
    },  
            
    main : function(){
        Pizza.player.loop(Pizza.canvas[0]); 
        Pizza.player.loop(Pizza.canvas[1]); 
        Pizza.player.loop(Pizza.canvas[2]);
    },
                
    /* Canvas */
    setCanvasObject : function(){
        this.canvas = new Array();
        this.canvas.push($("game_canvas"));
        this.context = this.canvas[0].getContext("2d");
    },
            
    setCanvasGame : function(){
        this.canvas = new Array();
        this.canvas.push($('canvas1'));
        this.canvas.push($('canvas2'));
        this.canvas.push($('canvas3'));
        this.context = null;
    },
            
     removeCursor : function(){
        $('game').style.cursor = "none";
    },
         
     restoreCursor : function(){
        $('game').style.cursor = "auto";
    },
            
     pointerCursor : function(){
        $('game').style.cursor = "pointer";
    },
            
     gameCursor : function(){
        $('player_canvas').style.cursor = "url("+GAME_CURSOR+"), auto";
     },
            
     hideLoaderCanvas: function(){
        $('loader_canvas').style.display = "none";
     },
             
     showLoaderCanvas: function(){
        $('loader_canvas').style.display = "block";
     },
            
     showPlayerCanvas: function(){
        $('player_canvas').style.display = "block";
     },
        
     hidePlayerCanvas: function(){
        $('player_canvas').style.display = "none";
     },
             
     clearCanvas : function(){
        this.context.clearRect(0,0,this.canvas[0].width, this.canvas[0].height);
    },
     
     clearTimer : function(){
        clearTimeout(this.timerID);
     },
      
     setTimer : function(t){
        this.timerID = t;
     },

      mouseMoveAction : function(e){
        Pizza.updateMouseLocation(e);
        if(Pizza.loader.isActive()){
            if(Pizza.loader.getPlayButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                Pizza.pointerCursor();
            }
            else
                Pizza.restoreCursor();
        }
        else{
            if(Pizza.loader.isLevel()){
                if(Pizza.loader.getEasyButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY )) ||
                   Pizza.loader.getHardButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                        Pizza.pointerCursor();
                }
                else
                    Pizza.restoreCursor();
             }
             else{
                 if(Pizza.loader.isMenu()){
                     if(Pizza.loader.getMenuButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                        Pizza.pointerCursor();
                      }
                 else
                    Pizza.restoreCursor();
                }
             }
        }
      },
            
      clickAction : function(e){
  
        if( Pizza.player ){
            if( Pizza.player.hitImages(new Coordinate( Pizza.mouseX, Pizza.mouseY ), Pizza.canvas_index ) ){
                Pizza.playClickSound();             
                if(Pizza.player.getLevel().checkWin()){
                    /*
                     * WIN the game.
                     */
                    Pizza.stopRunTime();
                    Pizza.player.getLevel().drawWin();
                    Pizza.pauseGameSound();
                    Pizza.playWinSound();
                    Pizza.showOverImage("you_win",4000);
                    setTimeout(function(){
                        Pizza.showMenu();
                     },4000);
                }
            }
            else{
                Pizza.playFailSound();
                Pizza.showOverImage("error_bg",300);
                Pizza.checkLife();
            }
        }
        if(Pizza.loader.isActive()){
            if(Pizza.loader.getPlayButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                Pizza.loader.setActive(false);
                Pizza.removeCursor();
                Pizza.clearCanvas();
                Pizza.loader.playVideo(); 
            }
        }
        else{
             if(Pizza.loader.isLevel()){
                 /*
                  * Choose the difficulty of the game.
                  */
                 if(Pizza.loader.getEasyButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                     Pizza.difficulty = GAME_LEVEL_EASY;
                     Pizza.initPlayer();
                     Pizza.startPlayer();
                 }
                 else if(Pizza.loader.getHardButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                     Pizza.difficulty = GAME_LEVEL_HARD;
                     Pizza.initPlayer();
                     Pizza.restoreCursor();
                     Pizza.loader.slideElement(Pizza.player.getLevel().toArray().slice());  
                 }   
             }
             else{
                 if(Pizza.loader.isMenu()){
                     if(Pizza.loader.getMenuButton().isOnButton(new Coordinate( Pizza.mouseX, Pizza.mouseY ))){
                        Pizza.loader.setMenu(false);
                        Pizza.showDifficulty();
                     }
                 }
             }
        }
    },
                   
      load : function(){
        this.loader = new Loader(this.canvas[0]);
        this.loader.drawPlayButton();
        this.loader.setActive(true);
     },
             
     showDifficulty : function(){
       this.playGameSound();
       this.initLife();
       this.loader.drawDifficultButton();
       this.loader.setLevel(true);
       this.restoreCursor(); 
     },
             
     showMenu : function(){
        Pizza.hidePlayerCanvas();
        Pizza.setCanvasObject();
        Pizza.clearCanvas();
        Pizza.loader.drawLevelButton();
        Pizza.loader.setLevel(false);
        Pizza.loader.setMenu(true);
        Pizza.player = null;                 
        Pizza.showLoaderCanvas();   
        Pizza.playGameSound();
     },
             
     initPlayer : function(){
       
        Pizza.hideLoaderCanvas();
        Pizza.setCanvasGame(); 
        Pizza.loader.setLevel(false);
        Pizza.player = new Player(Pizza.canvas);
        var level = new Level(Pizza.difficulty);
        level.randomLevel();
        Pizza.player.setLevel(level);
        Pizza.player.loadImage();
        Pizza.player.draw();                
        Pizza.gameCursor();
     },
             
     startPlayer : function(){
        Pizza.player.getLevel().draw();
        Pizza.showPlayerCanvas();
        Pizza.startRuntime();
     },
             
     showOverImage : function(img, time){
 
        var bg = $(img);
        var rect = $("game").getBoundingClientRect();
        bg.style.top = parseInt(rect.top - document.scrollTop)+"px";
        bg.style.left = parseInt(rect.left - document.scrollLeft)+"px";
        bg.style.display = "block";
        setTimeout(function(){bg.style.display = "none";},time);
 
     },
             
     initLife : function(){
        this.life = new Array($("life_1"),$("life_2"));
        for(var i = 0; i < this.life.length; i++){
            this.life[i].style.display = "inline";
        }
     },
             
     checkLife : function(){
        
         var l = this.life.pop();
         l.style.display = "none";
         if(!this.life.length){  
            /*
             * Lose the game.
             */
             Pizza.stopRunTime();
             Pizza.player.getLevel().drawLose();
             Pizza.pauseGameSound();
             Pizza.playLoseSound();
             Pizza.showOverImage("game_over", 4000);
             setTimeout(function(){
                 Pizza.showMenu();
             },4000);
         }
     },  
             
     /*****Event listeners ****************
       * 
       * 
       *************************************/
     attachEventListeners : function(){
          var _this = this;  
          for(var t = 0; t < _this.canvas.length; t++) {
            var cvs = _this.canvas[t];
            if(cvs.addEventListener){   
                cvs.addEventListener("mousemove", _this.mouseMoveAction , "false");
                cvs.addEventListener("click", _this.clickAction , "false");
            }
            else if(_this.attachEvent){
                cvs.attachEvent("onmouseenter", _this.mouseMoveAction);
                cvs.attachEvent("onclick", _this.clickAction);
            }
          }
    },
            
      removeEventListeners : function(){
          var _this = this;  
          for(var t = 0; t < _this.canvas.length; t++) {
            var cvs = _this.canvas[t];
            if(cvs.removeEventListener){   
                cvs.removeEventListener("mousemove", _this.mouseMoveAction , "false");
                cvs.removeEventListener("click", _this.clickAction , "false");
            }
            else if(_this.attachEvent){
                cvs.detachEvent("onmouseenter", _this.mouseMoveAction);
                cvs.detachEvent("onclick", _this.clickAction);
            }
          }
    },
            
     updateMouseLocation : function(e){
        e = e || window.event;
        var source = e.srcElement || e.currentTarget || e.fromElement || e.toElement;
        this.mouseX = ( e.pageX - parseInt(source.offsetLeft) );
        this.mouseY = ( e.pageY - parseInt(source.offsetTop) );
        this.canvas_index = parseInt(source.tabIndex);
    },
             
     playGameSound : function(){
 
            if(!this.gameSound){              
                this.gameSound = new Audio(GAME_SOUND);
                this.gameSound.volume = GAME_SOUND_VOLUME;
                    this.gameSound.addEventListener('ended', function() {
                    this.currentTime = 0;
                    this.play();
                }, false);
            }
            this.gameSound.play();
                      
     },
      
     pauseGameSound : function(){
            if(this.gameSound)
                this.gameSound.pause();
     },
             
     playClickSound : function(){
            this.clickSound = new Audio(CLICK_SOUND);
            this.clickSound.volume = CLICK_SOUND_VOLUME;
            this.clickSound.play();
     },
             
     playFailSound : function(){
            this.failSound = new Audio(FAIL_SOUND);
            this.failSound.volume = FAIL_SOUND_VOLUME;
            this.failSound.play();
     },
     
     playWinSound : function(){
            this.winSound = new Audio(WIN_SOUND);
            this.winSound.volume = WIN_SOUND_VOLUME;
            this.winSound.play();
     },
     
     playLoseSound : function(){
            this.winSound = new Audio(LOSE_SOUND);
            this.winSound.volume = LOSE_SOUND_VOLUME;
            this.winSound.play();
     }
};